#include <stdlib.h>
#include <process.h>
#include "archiverfe.h"

using namespace win32fe;

int archiver::Parse(void) {
  if (!tool::Parse()) {
    LI i = arg.begin();
    archivearg.push_front(*i++);
    arg.pop_front();
    while (i !=arg.end()) {
      std::string temp = *i;
      if (temp[0]=='-') {
        FoundFlag(i);
      } else {
        FoundFile(i);
      }
      i++;
      arg.pop_front();
    }
  } else {
    return(-1);
  }
  return(0);
}

int archiver::Execute(void) {
  int ierr = 0;
  ierr = tool::Execute();
  if ((!ierr) && !(helpfound || versionfound)) {
    std::string archivename = file.front();
    file.pop_front();
    RemoveQuotes(archivename);
    if (!GetShortPath(archivename)) {
      /* Failure of GetShortPath means the archive does not exist yet. */
      /* We must then check if the directory into which we place the archive exists. */
      std::string dir = "";
      std::string::size_type n = archivename.rfind("\\");
      if (n==std::string::npos) {
        /* Not a Windows path. */
        n = archivename.rfind("/");
        if (n==std::string::npos) {
          /* Nor a cygwin path.  It must go in the current directory. */
          char directory[MAX_PATH];                 /* Windows.h */
          GetCurrentDirectory(MAX_PATH,directory);  /* Windows.h */
          dir=(std::string)directory;
          GetShortPath(dir);
          archivename = dir + "\\" + archivename;
        }
      }
      if (n!=std::string::npos) {
        /* Either a cygwin path or a Windows path. */
        /* archivename[n] is either / or \, so replace it with \ */
        dir = archivename.substr(0,n);
        if (GetShortPath(dir)) {
          archivename = dir + "\\" + archivename.substr(n+1);
        } else {
          /* Bad News.  Perhaps the tool is smart enough to handle this case?! */
          if (!woff) {
            std::cerr << "Warning: win32fe: Directory not found: ";
            std::cerr << archivename.substr(0,n) << std::endl;
          }
        }
      }
    }
    archivearg.push_back(archivename);
    ierr = Archive();
  }
  return(ierr);
}

int archiver::Archive(void) {
  int ierr = 0;
  LI li = archivearg.begin();
  std::string header = *li++;
  Merge(header,archivearg,li);
  li = file.begin();
  std::string archive;
  while ((li != file.end()) && (!ierr)) {
    /* Invoke archiver several times to limit arg length <512 chars */
    archive = header;
    Merge(archive,file,li);
    if (verbose) {
      std::cout << archive << std::endl;
    }
    ierr = system(archive.c_str());
  }
  return(ierr);
}

void archiver::Help(void) {
  tool::Help();
  std::cout << "For archivers:" << std::endl;
  std::cout << "  The first file specified will be the archive name." << std::endl;
  std::cout << "  All subsequent files will be inserted into the archive." << std::endl << std::endl;
  std::cout << "Ex: win32fe tlib -u libfoo.lib foo.o bar.o" << std::endl << std::endl;
  std::cout << "Note: Long lists of files are automatically split into smaller lists." << std::endl << std::endl;
  std::cout << "=========================================================================" << std::endl << std::endl;
  
}

void archiver::FoundFlag(LI &i) {
  std::string temp = *i;
  if (temp == "-help") {
    helpfound = true;
  } else {
    archivearg.push_back(*i);
  }
}

void archiver::Merge(std::string &str,std::list<std::string> &liststr,LI &i) {
  std::string::size_type len=str.length();
  std::string::size_type maxlen=512;
  while (i!=liststr.end()) {
    std::string trystr=*i;
    if ((len+trystr.length()+1) < maxlen) {
      str += " " + trystr;
      len = str.length();
      i++;
    } else {
      break;
    }
  }
}

void archiver::DisplayVersion(void) {
  tool::DisplayVersion();
  std::string version = archivearg.front();
  version += " 2>&1 | head -1";
  system(version.c_str());
}
 
bool archiver::IsAKnownTool(void) {
  return(true);
}
 
