#include <stdlib.h>
#include <Windows.h>
#include "iclfe.h"

using namespace win32fe;

void icl::FindInstallation(void) {
  compiler::FindInstallation();

  /* Intel Compiler is currently only compatible with VS6.0, Visual C/C++ 98 */
  VSVersion = "VC98";
  if (verbose) {
    std::cout << "win32fe: Visual C++ Version: " << VSVersion << std::endl;
  }

  /* We must also find the Visual Studio Installation */
  /* Microsoft stashes this info in the registry */
  HKEY key;
  std::string vskey="Software\\Microsoft\\VisualStudio\\6.0";
  long ierr = RegOpenKeyEx(HKEY_LOCAL_MACHINE,vskey.c_str(),NULL,KEY_QUERY_VALUE,&key);
  if (ierr==ERROR_SUCCESS) {
    unsigned long length=MAX_PATH*sizeof(char);
    unsigned char buff[MAX_PATH];
    VisualStudioDir = "InstallDir";
    long ierr = RegQueryValueEx(key,VisualStudioDir.c_str(),NULL,NULL,buff,&length);
    if (ierr==ERROR_SUCCESS) {
      /* buff = VisualStudioDir\Common\IDE\IDE98 */
      VisualStudioDir = (std::string)((char *)buff);
      for (int i=0;i<3;i++) {
        std::string::size_type n = VisualStudioDir.rfind("\\");
        VisualStudioDir = VisualStudioDir.substr(0,n);
      }
      VisualStudioDir += "\\";
      GetShortPath(VisualStudioDir);
      if (verbose) {
        std::string longpath;
        GetLongPath(VisualStudioDir,longpath);
        std::cout << "win32fe: Visual Studio Installation: " << longpath << std::endl;
      }
    } else {
      if (!woff) {
        std::cout << "Warning: win32fe Cannot Find Visual Studio Installation" << std::endl;
        std::cout << "         Registry Key could not be queried." << std::endl;
      }
    }
    RegCloseKey(key);
  } else {
    if (!woff) {
      std::cout << "Warning: win32fe Cannot Find Visual Studio Installation" << std::endl;
      std::cout << "         Registry Key could not be found." << std::endl;
    }
  }
}

void icl::AddSystemInclude(void) {
  compiler::AddSystemInclude();
  /* Also requires Visual C/C++ headers */
  std::string include = VisualStudioDir + "VC98\\include";
  if (verbose) {
    std::string longpath;
    GetLongPath(include,longpath);
    std::cout << "win32fe: Adding Flag -I" << longpath << std::endl;
  }
  arg.push_back("-I" + include);
  LI i = arg.end();
  FoundI(--i);
  arg.pop_back();
}

void icl::AddSystemLib(void) {
  compiler::AddSystemLib();
  /* Also requires Visual C/C++ libraries */
  std::string lib = VisualStudioDir + "VC98\\lib";
  if (verbose) {
    std::string longpath;
    GetLongPath(lib,longpath);
    std::cout << "win32fe: Adding Flag -L" << longpath << std::endl;
  }
  arg.push_back("-L" + lib);
  LI i = arg.end();
  FoundL(--i);
  arg.pop_back();
}
